@@ -18,34 +18,22 @@ Tile retrieval (WMS, TMS, etc.).
 """
 import sys
 import time
-import warnings
+from typing import Any
 
 from mapproxy.version import version
 from mapproxy.image import ImageSource
 from mapproxy.util.py import reraise_exception
 from mapproxy.client.log import log_request
-from mapproxy.compat import PY2
-from mapproxy.compat.modules import urlparse
 
-if PY2:
-    import urllib2
-    from urllib2 import URLError, HTTPError, HTTPCookieProcessor
-    import httplib
-    from cookielib import CookieJar
-else:
-    from urllib import request as urllib2
-    from urllib.error import URLError, HTTPError
-    from urllib.request import HTTPCookieProcessor
-    from http import client as httplib
-    from http.cookiejar import CookieJar
+from urllib import request as urllib2
+from urllib.error import URLError, HTTPError
+from urllib.request import HTTPCookieProcessor
+from http import client as httplib
+from http.cookiejar import CookieJar
 
 import socket
 import ssl
 
-supports_ssl_default_context = False
-if hasattr(ssl, 'create_default_context'):
-    # Python >=2.7.9 and >=3.4.0
-    supports_ssl_default_context = True
 
 class HTTPClientError(Exception):
     def __init__(self, arg, response_code=None, full_msg=None):
@@ -55,24 +43,15 @@ class HTTPClientError(Exception):
 
 
 def build_https_handler(ssl_ca_certs, insecure):
-    if supports_ssl_default_context:
-        # python >=2.7.9 and >=3.4 supports ssl context in
-        # HTTPSHandler use this
-        if insecure:
-            ctx = ssl.SSLContext(ssl.PROTOCOL_SSLv23)
-            ctx.verify_mode = ssl.CERT_NONE
-        elif ssl_ca_certs:
-            ctx = ssl.create_default_context(cafile=ssl_ca_certs)
-        else:
-            ctx = ssl.create_default_context()
-        return urllib2.HTTPSHandler(context=ctx)
+    if insecure:
+        ctx = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
+        ctx.check_hostname = False
+        ctx.verify_mode = ssl.CERT_NONE
+    elif ssl_ca_certs:
+        ctx = ssl.create_default_context(cafile=ssl_ca_certs)
     else:
-        if insecure:
-            return None
-        else:
-            connection_class = verified_https_connection_with_ca_certs(
-                ssl_ca_certs)
-            return VerifiedHTTPSHandler(connection_class=connection_class)
+        ctx = ssl.create_default_context()
+    return urllib2.HTTPSHandler(context=ctx)
 
 
 class VerifiedHTTPSConnection(httplib.HTTPSConnection):
@@ -98,11 +77,13 @@ class VerifiedHTTPSConnection(httplib.HTTPSConnection):
 
         # wrap the socket using verification with the root
         #    certs in self.ca_certs_path
-        self.sock = ssl.wrap_socket(sock,
-                                    self.key_file,
-                                    self.cert_file,
-                                    cert_reqs=ssl.CERT_REQUIRED,
-                                    ca_certs=self._ca_certs)
+        context = ssl.create_default_context(ssl.Purpose.SERVER_AUTH)
+        context.verify_mode = ssl.CERT_REQUIRED
+        context.load_cert_chain(certfile=self.cert_file, keyfile=self.key_file)
+        context.load_verify_locations(cafile=self._ca_certs)
+        context.minimum_version = ssl.TLSVersion.TLSv1_2
+
+        self.sock = context.wrap_socket(sock, server_hostname=self.host)
 
 
 def verified_https_connection_with_ca_certs(ca_certs):
@@ -131,6 +112,7 @@ class _URLOpenerCache(object):
     Caches and reuses opener if possible (i.e. if they share the same
     ssl_ca_certs).
     """
+
     def __init__(self):
         self._opener = {}
 
@@ -163,27 +145,28 @@ class _URLOpenerCache(object):
 
         return opener
 
+
 create_url_opener = _URLOpenerCache()
 
+
 class HTTPClient(object):
     def __init__(self, url=None, username=None, password=None, insecure=False,
                  ssl_ca_certs=None, timeout=None, headers=None, hide_error_details=False,
                  manage_cookies=False):
         self._timeout = timeout
-        if url and url.startswith('https'):
-            if insecure:
-                ssl_ca_certs = None
-            elif ssl_ca_certs is None and not supports_ssl_default_context:
-                    raise HTTPClientError('No ca_certs file set (http.ssl_ca_certs). '
-                        'Set file or disable verification with http.ssl_no_cert_checks option.')
+        if url and url.startswith('https') and insecure:
+            ssl_ca_certs = None
 
-        self.opener = create_url_opener(ssl_ca_certs, url, username, password, insecure=insecure, manage_cookies=manage_cookies)
+        self.opener = create_url_opener(ssl_ca_certs, url, username, password,
+                                        insecure=insecure, manage_cookies=manage_cookies)
         self.header_list = headers.items() if headers else []
         self.hide_error_details = hide_error_details
 
-    def open(self, url, data=None, method=None):
+    def open(self, url, data=None, method=None) -> Any:
         code = None
         result = None
+        start_time = time.time()
+        req = None
         try:
             req = urllib2.Request(url, data=data)
         except ValueError as e:
@@ -192,9 +175,8 @@ class HTTPClient(object):
         for key, value in self.header_list:
             req.add_header(key, value)
         if method:
-            req.method=method
+            req.method = method
         try:
-            start_time = time.time()
             if self._timeout is not None:
                 result = self.opener.open(req, timeout=self._timeout)
             else:
@@ -248,6 +230,7 @@ class HTTPClient(object):
                 response_code=response_code,
             )
 
+
 def auth_data_from_url(url):
     """
     >>> auth_data_from_url('invalid_url')
@@ -301,6 +284,7 @@ def open_url(url):
     http_client = HTTPClient(url, username, password)
     return http_client.open(url)
 
+
 def retrieve_image(url, client=None):
     """
     Retrive an image from `url`.
@@ -312,4 +296,3 @@ def retrieve_image(url, client=None):
     if not resp.headers['content-type'].startswith('image'):
         raise HTTPClientError('response is not an image: (%s)' % (resp.read()))
     return ImageSource(resp)
-
