@@ -15,11 +15,11 @@
 # limitations under the License.
 
 import copy
-from mapproxy.compat import string_type
+
 
 class ImageOptions(object):
     def __init__(self, mode=None, transparent=None, opacity=None, resampling=None,
-        format=None, bgcolor=None, colors=None, encoding_options=None):
+                 format=None, bgcolor=None, colors=None, encoding_options=None):
         self.transparent = transparent
         self.opacity = opacity
         self.resampling = resampling
@@ -41,9 +41,25 @@ class ImageOptions(object):
                 options.append('%s=%r' % (k, v))
         return 'ImageOptions(%s)' % (', '.join(options), )
 
+    def __eq__(self, other):
+        if not isinstance(other, ImageOptions):
+            return NotImplemented
+
+        return (
+            self.transparent == other.transparent
+            and self.opacity == other.opacity
+            and self.resampling == other.resampling
+            and self.format == other.format
+            and self.mode == other.mode
+            and self.bgcolor == other.bgcolor
+            and self.colors == other.colors
+            and self.encoding_options == other.encoding_options
+        )
+
     def copy(self):
         return copy.copy(self)
 
+
 class ImageFormat(str):
     def __new__(cls, value, *args, **keywargs):
         if isinstance(value, ImageFormat):
@@ -67,7 +83,7 @@ class ImageFormat(str):
         return ext.strip()
 
     def __eq__(self, other):
-        if isinstance(other, string_type):
+        if isinstance(other, str):
             other = ImageFormat(other)
         elif not isinstance(other, ImageFormat):
             return NotImplemented
@@ -80,6 +96,7 @@ class ImageFormat(str):
     def __ne__(self, other):
         return not (self == other)
 
+
 def create_image(size, image_opts=None):
     """
     Create a new image that is compatible with the given `image_opts`.
@@ -100,7 +117,7 @@ def create_image(size, image_opts=None):
 
         bgcolor = image_opts.bgcolor or (255, 255, 255)
 
-        if isinstance(bgcolor, string_type):
+        if isinstance(bgcolor, str):
             bgcolor = ImageColor.getrgb(bgcolor)
 
         if image_opts.transparent and len(bgcolor) == 3:
@@ -126,6 +143,7 @@ class ImageFormats(object):
             opts = ImageOptions(transparent=False, format=format)
         return opts
 
+
 def compatible_image_options(img_opts, base_opts=None):
     """
     Return ImageOptions that is compatible with all given `img_opts`.
@@ -138,10 +156,10 @@ def compatible_image_options(img_opts, base_opts=None):
 
     transparent = None
     for o in img_opts:
-        if o.transparent == False:
+        if o.transparent is False:
             transparent = False
             break
-        if o.transparent == True:
+        if o.transparent is True:
             transparent = True
 
     if any(True for o in img_opts if o.mode):
@@ -164,4 +182,4 @@ def compatible_image_options(img_opts, base_opts=None):
         options.transparent = transparent
         options.mode = mode
 
-    return options
\ No newline at end of file
+    return options
