@@ -12,7 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
 """
 Retrieve maps/information from WMS servers.
 """
@@ -20,7 +19,7 @@ import sys
 from mapproxy.request.base import split_mime_type
 from mapproxy.cache.legend import Legend, legend_identifier
 from mapproxy.image import make_transparent, ImageSource, SubImageSource, bbox_position_in_image
-from mapproxy.image.merge import concat_legends
+from mapproxy.image.merge import concat_legends, concat_json_legends
 from mapproxy.image.transform import ImageTransformer
 from mapproxy.layer import MapExtent, DefaultMapExtent, BlankImage, LegendQuery, MapQuery, MapLayer
 from mapproxy.source import InfoSource, SourceError, LegendSource
@@ -30,8 +29,10 @@ from mapproxy.util.py import reraise_exception
 import logging
 log = logging.getLogger('mapproxy.source.wms')
 
+
 class WMSSource(MapLayer):
     supports_meta_tiles = True
+
     def __init__(self, client, image_opts=None, coverage=None, res_range=None,
                  transparent_color=None, transparent_color_tolerance=None,
                  supported_srs=None, supported_formats=None, fwd_req_params=None,
@@ -107,7 +108,8 @@ class WMSSource(MapLayer):
         if self.supported_formats and format not in self.supported_formats:
             format = self.supported_formats[0]
         if self.supported_srs:
-            # srs can be equal while still having a different srs_code (EPSG:3857/900913), make sure to use a supported srs_code
+            # srs can be equal while still having a different srs_code (EPSG:3857/900913),
+            # make sure to use a supported srs_code
             request_srs = None
             for srs in self.supported_srs:
                 if query.srs == srs:
@@ -143,7 +145,7 @@ class WMSSource(MapLayer):
         if xres < yres:
             src_size = dst_size[0], int(dst_size[0]/ratio + 0.5)
         else:
-            src_size = int(dst_size[1]*ratio +0.5), dst_size[1]
+            src_size = int(dst_size[1]*ratio + 0.5), dst_size[1]
 
         src_query = MapQuery(src_bbox, src_size, src_srs, format, dimensions=query.dimensions)
 
@@ -154,7 +156,7 @@ class WMSSource(MapLayer):
             img = ImageSource(resp, size=src_size, image_opts=self.image_opts)
 
         img = ImageTransformer(src_srs, dst_srs).transform(img, src_bbox,
-            query.size, dst_bbox, self.image_opts)
+                                                           query.size, dst_bbox, self.image_opts)
 
         img.format = format
         return img
@@ -181,9 +183,8 @@ class WMSSource(MapLayer):
         if self.coverage != other.coverage:
             return False
 
-
         if (query.dimensions_for_params(self.fwd_req_params) !=
-            query.dimensions_for_params(other.fwd_req_params)):
+                query.dimensions_for_params(other.fwd_req_params)):
             return False
 
         return True
@@ -197,14 +198,15 @@ class WMSSource(MapLayer):
             return None
 
         return WMSSource(client, image_opts=self.image_opts,
-            transparent_color=self.transparent_color,
-            transparent_color_tolerance=self.transparent_color_tolerance,
-            supported_srs=self.supported_srs,
-            supported_formats=self.supported_formats,
-            res_range=None, # layer outside res_range should already be filtered out
-            coverage=self.coverage,
-            fwd_req_params=self.fwd_req_params,
-        )
+                         transparent_color=self.transparent_color,
+                         transparent_color_tolerance=self.transparent_color_tolerance,
+                         supported_srs=self.supported_srs,
+                         supported_formats=self.supported_formats,
+                         res_range=None,  # layer outside res_range should already be filtered out
+                         coverage=self.coverage,
+                         fwd_req_params=self.fwd_req_params,
+                         )
+
 
 class WMSInfoSource(InfoSource):
     def __init__(self, client, fi_transformer=None, coverage=None):
@@ -243,7 +245,8 @@ class WMSLegendSource(LegendSource):
             legend = Legend(id=self.identifier, scale=None)
         else:
             legend = Legend(id=self.identifier, scale=query.scale)
-        if not self._cache.load(legend):
+
+        if not self._cache.load(legend) or query.format == 'json':
             legends = []
             error_occured = False
             for client in self.clients:
@@ -257,9 +260,14 @@ class WMSLegendSource(LegendSource):
                     # TODO errors?
                     log.error(e.args[0])
             format = split_mime_type(query.format)[1]
+
+            if format == 'json':
+                return concat_json_legends(legends)
+
             legend = Legend(source=concat_legends(legends, format=format),
                             id=self.identifier, scale=query.scale)
             if not error_occured:
                 self._cache.store(legend)
-        return legend.source
 
+        # returns ImageSource
+        return legend.source
