@@ -52,7 +52,9 @@ class TestWMS(SysTest):
                 transparent="true",
             ),
         )
-        self.expected_base_path = "/service?SERVICE=WMS&REQUEST=GetMap&HEIGHT=256" "&SRS=EPSG%3A900913&styles=&VERSION=1.1.1&WIDTH=512" "&BBOX=-20037508.3428,0.0,20037508.3428,20037508.3428"
+        self.expected_base_path = "/service?SERVICE=WMS&REQUEST=GetMap&HEIGHT=256"\
+                                  "&SRS=EPSG%3A900913&styles=&VERSION=1.1.1&WIDTH=512"\
+                                  "&BBOX=-20037508.3428,0.0,20037508.3428,20037508.3428"
 
     def test_mixed_mode(self, app, cache_dir):
         req_format = "png"
@@ -95,7 +97,9 @@ class TestWMS(SysTest):
 class TestTMS(SysTest):
 
     def setup_method(self):
-        self.expected_base_path = "/service?SERVICE=WMS&REQUEST=GetMap&HEIGHT=256" "&SRS=EPSG%3A900913&styles=&VERSION=1.1.1&WIDTH=512" "&BBOX=-20037508.3428,-20037508.3428,20037508.3428,0.0"
+        self.expected_base_path = "/service?SERVICE=WMS&REQUEST=GetMap&HEIGHT=256"\
+                                  "&SRS=EPSG%3A900913&styles=&VERSION=1.1.1&WIDTH=512"\
+                                  "&BBOX=-20037508.3428,-20037508.3428,20037508.3428,0.0"
 
     def test_mixed_mode(self, app, cache_dir):
         with create_mixed_mode_img((512, 256)) as img:
@@ -145,7 +149,9 @@ class TestWMTS(SysTest):
                 transparent="True",
             ),
         )
-        self.expected_base_path = "/service?SERVICE=WMS&REQUEST=GetMap&HEIGHT=256" "&SRS=EPSG%3A900913&styles=&VERSION=1.1.1&WIDTH=512" "&BBOX=-20037508.3428,0.0,20037508.3428,20037508.3428"
+        self.expected_base_path = "/service?SERVICE=WMS&REQUEST=GetMap&HEIGHT=256"\
+                                  "&SRS=EPSG%3A900913&styles=&VERSION=1.1.1&WIDTH=512"\
+                                  "&BBOX=-20037508.3428,0.0,20037508.3428,20037508.3428"
 
     def test_mixed_mode(self, app, cache_dir):
         with create_mixed_mode_img((512, 256)) as img:
