@@ -16,8 +16,6 @@
 
 from __future__ import division
 
-import functools
-
 from io import BytesIO
 
 import pytest
@@ -47,7 +45,7 @@ TEST_TILE = create_tmp_image((256, 256))
 
 class TestMultiCacheLayer(SysTest):
 
-    def setup(self):
+    def setup_method(self):
         self.common_cap_req = WMTS100CapabilitiesRequest(
             url="/service?",
             param=dict(service="WMTS", version="1.0.0", request="GetCapabilities"),
@@ -68,6 +66,10 @@ class TestMultiCacheLayer(SysTest):
             ),
         )
 
+    def test_image_config(self, app):
+        resp = app.get("/tms/1.0.0/")
+        assert "http://localhost/tms/1.0.0/cache_image/EPSG25832" in resp
+
     def test_tms_capabilities(self, app):
         resp = app.get("/tms/1.0.0/")
         assert "http://localhost/tms/1.0.0/multi_cache/EPSG25832" in resp
@@ -76,7 +78,7 @@ class TestMultiCacheLayer(SysTest):
         assert "http://localhost/tms/1.0.0/multi_cache/EPSG31467" in resp
         assert "http://localhost/tms/1.0.0/cache/EPSG25832" in resp
         xml = resp.lxml
-        assert xml.xpath("count(//TileMap)") == 5
+        assert xml.xpath("count(//TileMap)") == 8
 
     def test_wmts_capabilities(self, app):
         req = str(self.common_cap_req)
@@ -89,7 +91,7 @@ class TestMultiCacheLayer(SysTest):
         )
         assert set(
             xml.xpath("//wmts:Layer/ows:Identifier/text()", namespaces=ns_wmts)
-        ) == set(["cache", "multi_cache"])
+        ) == set(["cache", "multi_cache", "cache_image"])
         assert set(
             xml.xpath(
                 "//wmts:Contents/wmts:TileMatrixSet/ows:Identifier/text()",
@@ -111,13 +113,13 @@ class TestMultiCacheLayer(SysTest):
         )
 
         layer_names = set(xml.xpath("//Layer/Layer/Name/text()"))
-        expected_names = set(["wms_only", "cache"])
+        expected_names = set(["wms_only", "cache", "cache_image"])
         assert layer_names == expected_names
 
     def test_get_tile_webmerc(self, app):
         serv = MockServ(42423, bbox_aware_query_comparator=True)
         serv.expects(
-            "/service?layers=foo,bar&width=256&version=1.1.1&bbox=-20037508.3428,0.0,0.0,20037508.3428&service=WMS&format=image%2Fpng&styles=&srs=EPSG%3A3857&request=GetMap&height=256"
+            "/service?layers=foo,bar&width=256&version=1.1.1&bbox=-20037508.3428,0.0,0.0,20037508.3428&service=WMS&format=image%2Fpng&styles=&srs=EPSG%3A3857&request=GetMap&height=256"  # noqa
         ).returns(TEST_TILE)
         with serv:
             resp = app.get(str(self.common_tile_req))
@@ -128,7 +130,7 @@ class TestMultiCacheLayer(SysTest):
     def test_get_tile_utm(self, app):
         serv = MockServ(42423, bbox_aware_query_comparator=True)
         serv.expects(
-            "/service?layers=foo,bar&width=256&version=1.1.1&bbox=-46133.17,5675047.40429,580038.965712,6301219.54&service=WMS&format=image%2Fpng&styles=&srs=EPSG%3A25832&request=GetMap&height=256"
+            "/service?layers=foo,bar&width=256&version=1.1.1&bbox=-46133.17,5675047.40429,580038.965712,6301219.54&service=WMS&format=image%2Fpng&styles=&srs=EPSG%3A25832&request=GetMap&height=256"  # noqa
         ).returns(TEST_TILE)
         self.common_tile_req.params["tilematrixset"] = "utm32"
 
@@ -142,22 +144,22 @@ class TestMultiCacheLayer(SysTest):
         serv = MockServ(42423, bbox_aware_query_comparator=True, unordered=True)
         # gk3 cache requests UTM tiles
         serv.expects(
-            "/service?layers=foo,bar&width=256&version=1.1.1&bbox=423495.931784,5596775.88732,501767.448748,5675047.40429&service=WMS&format=image%2Fpng&styles=&srs=EPSG%3A25832&request=GetMap&height=256"
+            "/service?layers=foo,bar&width=256&version=1.1.1&bbox=423495.931784,5596775.88732,501767.448748,5675047.40429&service=WMS&format=image%2Fpng&styles=&srs=EPSG%3A25832&request=GetMap&height=256"  # noqa
         ).returns(TEST_TILE)
         serv.expects(
-            "/service?layers=foo,bar&width=256&version=1.1.1&bbox=345224.41482,5596775.88732,423495.931784,5675047.40429&service=WMS&format=image%2Fpng&styles=&srs=EPSG%3A25832&request=GetMap&height=256"
+            "/service?layers=foo,bar&width=256&version=1.1.1&bbox=345224.41482,5596775.88732,423495.931784,5675047.40429&service=WMS&format=image%2Fpng&styles=&srs=EPSG%3A25832&request=GetMap&height=256"  # noqa
         ).returns(TEST_TILE)
         serv.expects(
-            "/service?layers=foo,bar&width=256&version=1.1.1&bbox=345224.41482,5518504.37036,423495.931784,5596775.88732&service=WMS&format=image%2Fpng&styles=&srs=EPSG%3A25832&request=GetMap&height=256"
+            "/service?layers=foo,bar&width=256&version=1.1.1&bbox=345224.41482,5518504.37036,423495.931784,5596775.88732&service=WMS&format=image%2Fpng&styles=&srs=EPSG%3A25832&request=GetMap&height=256"  # noqa
         ).returns(TEST_TILE)
         serv.expects(
-            "/service?layers=foo,bar&width=256&version=1.1.1&bbox=423495.931784,5518504.37036,501767.448748,5596775.88732&service=WMS&format=image%2Fpng&styles=&srs=EPSG%3A25832&request=GetMap&height=256"
+            "/service?layers=foo,bar&width=256&version=1.1.1&bbox=423495.931784,5518504.37036,501767.448748,5596775.88732&service=WMS&format=image%2Fpng&styles=&srs=EPSG%3A25832&request=GetMap&height=256"  # noqa
         ).returns(TEST_TILE)
         serv.expects(
-            "/service?layers=foo,bar&width=256&version=1.1.1&bbox=345224.41482,5440232.8534,423495.931784,5518504.37036&service=WMS&format=image%2Fpng&styles=&srs=EPSG%3A25832&request=GetMap&height=256"
+            "/service?layers=foo,bar&width=256&version=1.1.1&bbox=345224.41482,5440232.8534,423495.931784,5518504.37036&service=WMS&format=image%2Fpng&styles=&srs=EPSG%3A25832&request=GetMap&height=256"  # noqa
         ).returns(TEST_TILE)
         serv.expects(
-            "/service?layers=foo,bar&width=256&version=1.1.1&bbox=423495.931784,5440232.8534,501767.448748,5518504.37036&service=WMS&format=image%2Fpng&styles=&srs=EPSG%3A25832&request=GetMap&height=256"
+            "/service?layers=foo,bar&width=256&version=1.1.1&bbox=423495.931784,5440232.8534,501767.448748,5518504.37036&service=WMS&format=image%2Fpng&styles=&srs=EPSG%3A25832&request=GetMap&height=256"  # noqa
         ).returns(TEST_TILE)
         self.common_tile_req.params["tilematrixset"] = "gk3"
         with serv:
