@@ -17,7 +17,7 @@ from __future__ import print_function
 
 import yaml
 
-from mapproxy.config.validator import validate_references
+from mapproxy.config.validator import validate
 
 
 class TestValidator(object):
@@ -49,18 +49,28 @@ class TestValidator(object):
     def test_valid_config(self):
         conf = self._test_conf()
 
-        errors = validate_references(conf)
+        errors = validate(conf)
         assert errors == []
 
     def test_missing_layer_source(self):
         conf = self._test_conf()
         del conf['caches']['one_cache']
 
-        errors = validate_references(conf)
+        errors = validate(conf)
         assert errors == [
             "Source 'one_cache' for layer 'one' not in cache or source section"
         ]
 
+    def test_multiple_errors(self):
+        conf = self._test_conf()
+        del conf['caches']['one_cache']
+        del conf['services']
+        errors = validate(conf)
+        assert errors == [
+            "'services' is a required property in root",
+            "Source 'one_cache' for layer 'one' not in cache or source section",
+        ]
+
     def test_empty_layer_sources(self):
         conf = self._test_conf('''
             layers:
@@ -69,16 +79,21 @@ class TestValidator(object):
                   sources: []
         ''')
 
-        errors = validate_references(conf)
-        assert errors == [
-            "Missing sources for layer 'one'"
+        errors = validate(conf)
+        assert errors in [
+            [
+                "[] is too short in root.layers[0].sources"
+            ],
+            [
+                "[] should be non-empty in root.layers[0].sources"
+            ]
         ]
 
     def test_missing_cache_source(self):
         conf = self._test_conf()
         del conf['sources']['one_source']
 
-        errors = validate_references(conf)
+        errors = validate(conf)
         assert errors == [
             "Source 'one_source' for cache 'one_cache' not found in config"
         ]
@@ -87,17 +102,17 @@ class TestValidator(object):
         conf = self._test_conf()
         del conf['layers']
 
-        errors = validate_references(conf)
+        errors = validate(conf)
         assert errors == [
-            'Missing layers section'
+            "'layers' is a required property in root"
         ]
 
     def test_missing_services_section(self):
         conf = self._test_conf()
         del conf['services']
-        errors = validate_references(conf)
+        errors = validate(conf)
         assert errors == [
-            'Missing services section'
+            "'services' is a required property in root"
         ]
 
     def test_tile_source(self):
@@ -107,7 +122,7 @@ class TestValidator(object):
                   tile_sources: [missing]
         ''')
 
-        errors = validate_references(conf)
+        errors = validate(conf)
         assert errors == [
             "Tile source 'missing' for layer 'one' not in cache section"
         ]
@@ -117,12 +132,13 @@ class TestValidator(object):
             caches:
                 one_cache:
                     grids: [MYGRID_OTHERGRID]
+                    sources: []
             grids:
                 MYGRID:
                     base: GLOBAL_GEODETIC
         ''')
 
-        errors = validate_references(conf)
+        errors = validate(conf)
         assert errors == [
             "Grid 'MYGRID_OTHERGRID' for cache 'one_cache' not found in config"
         ]
@@ -132,7 +148,7 @@ class TestValidator(object):
 
         del conf['sources']['one_source']['req']['layers']
 
-        errors = validate_references(conf)
+        errors = validate(conf)
         assert errors == [
             "Missing 'layers' for source 'one_source'"
         ]
@@ -148,21 +164,21 @@ class TestValidator(object):
                         binary: /foo/bar/baz
         ''')
 
-        errors = validate_references(conf)
+        errors = validate(conf)
         assert errors == [
             'Could not find mapserver binary (/foo/bar/baz)'
         ]
 
         del conf['sources']['one_source']['mapserver']['binary']
 
-        errors = validate_references(conf)
+        errors = validate(conf)
         assert errors == [
             "Missing mapserver binary for source 'one_source'"
         ]
 
         del conf['sources']['one_source']['mapserver']
 
-        errors = validate_references(conf)
+        errors = validate(conf)
         assert errors == [
             "Missing mapserver binary for source 'one_source'"
         ]
@@ -179,14 +195,14 @@ class TestValidator(object):
                     binary: /foo/bar/baz
         ''')
 
-        errors = validate_references(conf)
+        errors = validate(conf)
         assert errors == [
             'Could not find mapserver binary (/foo/bar/baz)'
         ]
 
         del conf['globals']['mapserver']['binary']
 
-        errors = validate_references(conf)
+        errors = validate(conf)
         assert errors == [
             "Missing mapserver binary for source 'one_source'"
         ]
@@ -199,12 +215,39 @@ class TestValidator(object):
                     sources: ['one_source:foo,bar']
         ''')
 
-        errors = validate_references(conf)
+        errors = validate(conf)
         assert errors == [
             "Supported layers for source 'one_source' are 'one' but tagged source "
             "requested layers 'foo, bar'"
         ]
 
+    def test_tagged_layer_sources_with_layers(self):
+        conf = self._test_conf('''
+            layers:
+                - name: one
+                  title: One
+                  sources: ['one_source:foo,bar']
+        ''')
+
+        errors = validate(conf)
+        assert errors == [
+            "Supported layers for source 'one_source' are 'one' but tagged source "
+            "requested layers 'foo, bar'"
+        ]
+
+    def test_tagged_layer_sources_without_layers(self):
+        conf = self._test_conf('''
+            layers:
+                - name: one
+                  title: One
+                  sources: ['one_source:foo,bar']
+        ''')
+
+        del conf['sources']['one_source']['req']['layers']
+
+        errors = validate(conf)
+        assert errors == []
+
     def test_tagged_source_without_layers(self):
         conf = self._test_conf('''
             caches:
@@ -215,7 +258,7 @@ class TestValidator(object):
 
         del conf['sources']['one_source']['req']['layers']
 
-        errors = validate_references(conf)
+        errors = validate(conf)
         assert errors == []
 
     def test_tagged_source_with_colons(self):
@@ -228,7 +271,7 @@ class TestValidator(object):
 
         del conf['sources']['one_source']['req']['layers']
 
-        errors = validate_references(conf)
+        errors = validate(conf)
         assert errors == []
 
     def test_with_grouped_layer(self):
@@ -242,7 +285,7 @@ class TestValidator(object):
                       sources: [one_cache]
         ''')
 
-        errors = validate_references(conf)
+        errors = validate(conf)
         assert errors == []
 
     def test_without_cache(self):
@@ -253,7 +296,7 @@ class TestValidator(object):
                 sources: [one_source]
         ''')
 
-        errors = validate_references(conf)
+        errors = validate(conf)
         assert errors == []
 
     def test_mapserver_with_tagged_layers(self):
@@ -272,7 +315,7 @@ class TestValidator(object):
                     sources: ['one_source:foo,bar']
         ''')
 
-        errors = validate_references(conf)
+        errors = validate(conf)
         assert errors == [
             'Could not find mapserver binary (/foo/bar/baz)',
             "Supported layers for source 'one_source' are 'one' but tagged source "
@@ -292,7 +335,7 @@ class TestValidator(object):
                     sources: ['one_source:foo,bar']
         ''')
 
-        errors = validate_references(conf)
+        errors = validate(conf)
         assert errors == [
             "Supported layers for source 'one_source' are 'one' but tagged source "
             "requested layers 'foo, bar'"
@@ -310,7 +353,7 @@ class TestValidator(object):
                     sources: ['one_source:foo,bar']
         ''')
 
-        errors = validate_references(conf)
+        errors = validate(conf)
         assert errors == [
             "Found tagged source 'one_source' in cache 'one_cache' but tagged sources "
             "only supported for 'wms, mapserver, mapnik' sources"
@@ -326,7 +369,7 @@ class TestValidator(object):
                     sources: ['one_source']
         ''')
 
-        errors = validate_references(conf)
+        errors = validate(conf)
         assert errors == []
 
     def test_with_int_0_as_names_and_layers(self):
@@ -351,7 +394,7 @@ class TestValidator(object):
                         layers: 0
         ''')
 
-        errors = validate_references(conf)
+        errors = validate(conf)
         assert errors == []
 
     def test_band_merge_missing_source(self):
@@ -382,7 +425,7 @@ class TestValidator(object):
                         layers: dop
         ''')
 
-        errors = validate_references(conf)
+        errors = validate(conf)
         assert errors == [
             "Source 'missing1' for cache 'one_cache' not found in config",
             "Source 'missing2' for cache 'cache_missing_source' not found in config",
