@@ -18,6 +18,7 @@ from io import BytesIO
 from mapproxy.compat.image import Image
 from mapproxy.exception import RequestError
 from mapproxy.request import url_decode
+from mapproxy.request.base import Request
 from mapproxy.request.wms import WMSMapRequest
 from mapproxy.request.wms.exception import (
     WMS100ExceptionHandler,
@@ -25,6 +26,7 @@ from mapproxy.request.wms.exception import (
     WMS130ExceptionHandler,
     WMS110ExceptionHandler,
 )
+from mapproxy.service.ows import OWSServer
 from mapproxy.test.helper import Mocker, validate_with_dtd, validate_with_xsd
 from mapproxy.test.image import is_png
 
@@ -37,6 +39,7 @@ REQUEST=GetMap&STYLES=&EXCEPTIONS=application%2Fvnd.ogc.se_xml&SRS=EPSG%3A900913
 BBOX=8,4,9,5&WIDTH=150&HEIGHT=100""".replace('\n', ''))
         self.req = req
 
+
 class TestWMS111ExceptionHandler(Mocker):
     def test_render(self):
         req = self.mock(WMSMapRequest)
@@ -56,10 +59,11 @@ class TestWMS111ExceptionHandler(Mocker):
 """
         assert expected_resp.strip() == response.data
         assert validate_with_dtd(response.data, 'wms/1.1.1/exception_1_1_1.dtd')
+
     def test_render_w_code(self):
         req = self.mock(WMSMapRequest)
         req_ex = RequestError('the exception message', code='InvalidFormat',
-                                  request=req)
+                              request=req)
         ex_handler = WMS111ExceptionHandler()
         self.expect(req.exception_handler).result(ex_handler)
 
@@ -76,6 +80,7 @@ class TestWMS111ExceptionHandler(Mocker):
         assert expected_resp.strip() == response.data
         assert validate_with_dtd(response.data, 'wms/1.1.1/exception_1_1_1.dtd')
 
+
 class TestWMS110ExceptionHandler(Mocker):
     def test_render(self):
         req = self.mock(WMSMapRequest)
@@ -95,10 +100,11 @@ class TestWMS110ExceptionHandler(Mocker):
 """
         assert expected_resp.strip() == response.data
         assert validate_with_dtd(response.data, 'wms/1.1.0/exception_1_1_0.dtd')
+
     def test_render_w_code(self):
         req = self.mock(WMSMapRequest)
         req_ex = RequestError('the exception message', code='InvalidFormat',
-                                  request=req)
+                              request=req)
         ex_handler = WMS110ExceptionHandler()
         self.expect(req.exception_handler).result(ex_handler)
 
@@ -115,6 +121,7 @@ class TestWMS110ExceptionHandler(Mocker):
         assert expected_resp.strip() == response.data
         assert validate_with_dtd(response.data, 'wms/1.1.0/exception_1_1_0.dtd')
 
+
 class TestWMS130ExceptionHandler(Mocker):
     def test_render(self):
         req = self.mock(WMSMapRequest)
@@ -137,10 +144,11 @@ http://schemas.opengis.net/wms/1.3.0/exceptions_1_3_0.xsd">
 """
         assert expected_resp.strip() == response.data
         assert validate_with_xsd(response.data, 'wms/1.3.0/exceptions_1_3_0.xsd')
+
     def test_render_w_code(self):
         req = self.mock(WMSMapRequest)
         req_ex = RequestError('the exception message', code='InvalidFormat',
-                                  request=req)
+                              request=req)
         ex_handler = WMS130ExceptionHandler()
         self.expect(req.exception_handler).result(ex_handler)
 
@@ -160,6 +168,91 @@ http://schemas.opengis.net/wms/1.3.0/exceptions_1_3_0.xsd">
         assert expected_resp.strip() == response.data
         assert validate_with_xsd(response.data, 'wms/1.3.0/exceptions_1_3_0.xsd')
 
+    def test_missing_service_request(self):
+        reqString = "REQUEST=GetCapabilities"
+        conf = {
+            'QUERY_STRING': reqString,
+            'wsgi.url_scheme': 'http',
+            'HTTP_HOST': 'localhost',
+        }
+        req = Request(conf)
+        ows_services = []
+        server = OWSServer(ows_services)
+        response = server.handle(req)
+
+        expected_resp = """
+<?xml version="1.0"?>
+<ows:ExceptionReport xmlns:ows="http://www.opengis.net/ows/1.1"
+  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
+  xsi:schemaLocation="http://www.opengis.net/ows/1.1 http://schemas.opengis.net/ows/1.1.0/owsExceptionReport.xsd"
+  version="1.0.0" xml:lang="en">
+  <ows:Exception exceptionCode="MissingParameterValue"
+      locator="service">
+    <ows:ExceptionText>The service parameter is missing</ows:ExceptionText>
+  </ows:Exception>
+</ows:ExceptionReport>
+"""
+
+        assert expected_resp.strip() == response.response.strip()
+        assert response.content_type == 'text/xml; charset=utf-8'
+        assert response.status == '400 Bad Request'
+        assert validate_with_xsd(response.response, 'ows/1.1.0/owsExceptionReport.xsd')
+
+    def test_invalid_service_request(self):
+        reqString = "REQUEST=GetCapabilities&SERVICE=wms"
+        conf = {
+            'QUERY_STRING': reqString,
+            'wsgi.url_scheme': 'http',
+            'HTTP_HOST': 'localhost',
+        }
+        req = Request(conf)
+        ows_services = []
+        server = OWSServer(ows_services)
+        response = server.handle(req)
+
+        expected_resp = """
+<?xml version="1.0"?>
+<ows:ExceptionReport xmlns:ows="http://www.opengis.net/ows/1.1"
+  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
+  xsi:schemaLocation="http://www.opengis.net/ows/1.1 http://schemas.opengis.net/ows/1.1.0/owsExceptionReport.xsd"
+  version="1.0.0" xml:lang="en">
+  <ows:Exception exceptionCode="InvalidParameterValue"
+      locator="service">
+    <ows:ExceptionText>The value of the service parameter &quot;wms&quot; is invalid</ows:ExceptionText>
+  </ows:Exception>
+</ows:ExceptionReport>
+"""
+
+        assert expected_resp.strip() == response.response
+        assert response.content_type == 'text/xml; charset=utf-8'
+        assert response.status == '400 Bad Request'
+        assert validate_with_xsd(response.response, 'ows/1.1.0/owsExceptionReport.xsd')
+
+    def test_valid_service_request(self):
+        reqString = "REQUEST=GetCapabilities&SERVICE=wms"
+        conf = {
+            'QUERY_STRING': reqString,
+            'wsgi.url_scheme': 'http',
+            'HTTP_HOST': 'localhost',
+        }
+        req = Request(conf)
+
+        class Service(object):
+
+            def __init__(self, service):
+                self.service = service
+
+            def handle(self, req):
+                return 'all good'
+
+        ows_services = [Service('wms')]
+        server = OWSServer(ows_services)
+        response = server.handle(req)
+
+        expected_resp = 'all good'
+        assert expected_resp == response
+
+
 class TestWMS100ExceptionHandler(Mocker):
     def test_render(self):
         req = self.mock(WMSMapRequest)
@@ -179,10 +272,11 @@ the exception message
 """
         assert expected_resp.strip() == response.data
 
+
 class TestWMSImageExceptionHandler(ExceptionHandlerTest):
     def test_exception(self):
         self.req.set('exceptions', 'inimage')
-        self.req.set('transparent', 'true' )
+        self.req.set('transparent', 'true')
 
         req = WMSMapRequest(self.req)
         req_ex = RequestError('the exception message', request=req)
@@ -193,9 +287,10 @@ class TestWMSImageExceptionHandler(ExceptionHandlerTest):
         assert is_png(data)
         img = Image.open(data)
         assert img.size == (150, 100)
+
     def test_exception_w_transparent(self):
         self.req.set('exceptions', 'inimage')
-        self.req.set('transparent', 'true' )
+        self.req.set('transparent', 'true')
 
         req = WMSMapRequest(self.req)
         req_ex = RequestError('the exception message', request=req)
@@ -224,8 +319,9 @@ class TestWMSBlankExceptionHandler(ExceptionHandlerTest):
         assert is_png(data)
         img = Image.open(data)
         assert img.size == (150, 100)
-        assert img.getpixel((0, 0)) == 0  #pallete image
+        assert img.getpixel((0, 0)) == 0  # pallete image
         assert img.getpalette()[0:3] == [255, 255, 255]
+
     def test_exception_w_bgcolor(self):
         self.req.set('exceptions', 'blank')
         self.req.set('bgcolor', '0xff00ff')
@@ -239,11 +335,12 @@ class TestWMSBlankExceptionHandler(ExceptionHandlerTest):
         assert is_png(data)
         img = Image.open(data)
         assert img.size == (150, 100)
-        assert img.getpixel((0, 0)) == 0  #pallete image
+        assert img.getpixel((0, 0)) == 0  # pallete image
         assert img.getpalette()[0:3] == [255, 0, 255]
+
     def test_exception_w_transparent(self):
         self.req.set('exceptions', 'blank')
-        self.req.set('transparent', 'true' )
+        self.req.set('transparent', 'true')
 
         req = WMSMapRequest(self.req)
         req_ex = RequestError('the exception message', request=req)
