@@ -22,13 +22,10 @@ import shutil
 import pytest
 import shapely
 import shapely.prepared
-try:
-    # shapely >=1.6
-    from shapely.errors import ReadingError
-except ImportError:
-    from shapely.geos import ReadingError
+from shapely.errors import ShapelyError
 
-from mapproxy.srs import SRS, bbox_equals
+from mapproxy.srs import SRS
+from mapproxy.util.bbox import bbox_equals
 from mapproxy.util.geom import (
     load_polygons,
     load_datasource,
@@ -100,6 +97,7 @@ INTERSECTING_POLYGONS = """POLYGON ((0 0, 10 0, 10 10, 0 10, 0 0))
 POLYGON ((5 0, 15 0, 15 10, 5 10, 5 0))
 """
 
+
 class TestPolygonLoading(object):
     def test_loading_polygon(self):
         with TempFile() as fname:
@@ -108,7 +106,7 @@ class TestPolygonLoading(object):
             polygon = load_polygons(fname)
             bbox, polygon = build_multipolygon(polygon, simplify=True)
             assert polygon.is_valid
-            assert polygon.type == 'Polygon'
+            assert polygon.geom_type == 'Polygon'
 
     def test_loading_multipolygon(self):
         with TempFile() as fname:
@@ -119,13 +117,13 @@ class TestPolygonLoading(object):
             polygon = load_polygons(fname)
             bbox, polygon = build_multipolygon(polygon, simplify=True)
             assert polygon.is_valid
-            assert polygon.type == 'MultiPolygon'
+            assert polygon.geom_type == 'MultiPolygon'
 
     def test_loading_broken(self):
         with TempFile() as fname:
             with open(fname, 'wb') as f:
                 f.write(b"POLYGON((")
-            with pytest.raises(ReadingError):
+            with pytest.raises(ShapelyError):
                 load_polygons(fname)
 
     def test_loading_skip_non_polygon(self):
@@ -136,7 +134,7 @@ class TestPolygonLoading(object):
             polygon = load_polygons(fname)
             bbox, polygon = build_multipolygon(polygon, simplify=True)
             assert polygon.is_valid
-            assert polygon.type == 'Polygon'
+            assert polygon.geom_type == 'Polygon'
 
     def test_loading_intersecting_polygons(self):
         # check that the self intersection is eliminated
@@ -147,19 +145,23 @@ class TestPolygonLoading(object):
             polygon = load_polygons(fname)
             bbox, polygon = build_multipolygon(polygon, simplify=True)
             assert polygon.is_valid
-            assert polygon.type == 'Polygon'
+            assert polygon.geom_type == 'Polygon'
             assert polygon.equals(shapely.geometry.Polygon([(0, 0), (15, 0), (15, 10), (0, 10)]))
 
 
 class TestGeoJSONLoading(object):
     @pytest.mark.parametrize('geojson,geometry', [
-        [ '''{"type": "Polygon", "coordinates": [[[0, 0], [10, 0], [10, 10], [0, 0]]]}''',
+        ['''{"type": "Polygon", "coordinates": [[[0, 0], [10, 0], [10, 10], [0, 0]]]}''',
             shapely.geometry.Polygon([[0, 0], [10, 0], [10, 10], [0, 0]]), ],
-        [ '''{"type": "MultiPolygon", "coordinates": [[[[0, 0], [10, 0], [10, 10], [0, 0]]], [[[20, 0], [30, 0], [20, 10], [20, 0]]]]}''',
-            shapely.geometry.Polygon([[0, 0], [10, 0], [10, 10], [0, 0]]).union(shapely.geometry.Polygon([[20, 0], [30, 0], [20, 10], [20, 0]])), ],
-        [ '''{"type": "Feature", "geometry": {"type": "Polygon", "coordinates": [[[0, 0], [10, 0], [10, 10], [0, 0]]]}}''',
+        ['''{"type": "MultiPolygon", "coordinates":
+            [[[[0, 0], [10, 0], [10, 10], [0, 0]]], [[[20, 0], [30, 0], [20, 10], [20, 0]]]]}''',
+            shapely.geometry.Polygon([[0, 0], [10, 0], [10, 10], [0, 0]]).union(
+                shapely.geometry.Polygon([[20, 0], [30, 0], [20, 10], [20, 0]])), ],
+        ['''{"type": "Feature", "geometry": {"type": "Polygon", "coordinates":
+            [[[0, 0], [10, 0], [10, 10], [0, 0]]]}}''',
             shapely.geometry.Polygon([[0, 0], [10, 0], [10, 10], [0, 0]]), ],
-        [ '''{"type": "FeatureCollection", "features": [{"type": "Feature", "geometry": {"type": "Polygon", "coordinates": [[[0, 0], [10, 0], [10, 10], [0, 0]]]}}]}''',
+        ['''{"type": "FeatureCollection", "features": [{"type": "Feature", "geometry":
+            {"type": "Polygon", "coordinates": [[[0, 0], [10, 0], [10, 10], [0, 0]]]}}]}''',
             shapely.geometry.Polygon([[0, 0], [10, 0], [10, 10], [0, 0]]), ],
     ])
     def test_geojson(self, geojson, geometry):
@@ -169,7 +171,7 @@ class TestGeoJSONLoading(object):
             polygon = load_geojson(fname)
             bbox, polygon = build_multipolygon(polygon, simplify=True)
             assert polygon.is_valid
-            assert polygon.type in ('Polygon', 'MultiPolygon'), polygon.type
+            assert polygon.geom_type in ('Polygon', 'MultiPolygon'), polygon.geom_type
             assert polygon.equals(geometry)
 
 
@@ -202,14 +204,15 @@ class TestTransform(object):
         with pytest.raises(ValueError):
             transform_geometry(SRS(4326), SRS(900913), p)
 
+
 class TestBBOXPolygon(object):
     def test_bbox_polygon(self):
         p = bbox_polygon([5, 53, 6, 54])
-        assert p.type == 'Polygon'
+        assert p.geom_type == 'Polygon'
 
 
 class TestGeomCoverage(object):
-    def setup(self):
+    def setup_method(self):
         # box from 10 10 to 80 80 with small spike/corner to -10 60 (upper left)
         self.geom = shapely.wkt.loads(
             "POLYGON((10 10, 10 50, -10 60, 10 80, 80 80, 80 10, 10 10))")
@@ -219,7 +222,7 @@ class TestGeomCoverage(object):
         assert bbox_equals(self.coverage.bbox, [-10, 10, 80, 80], 0.0001)
 
     def test_geom(self):
-        assert self.coverage.geom.type == 'Polygon'
+        assert self.coverage.geom.geom_type == 'Polygon'
 
     def test_contains(self):
         assert self.coverage.contains((15, 15, 20, 20), SRS(4326))
@@ -252,15 +255,16 @@ class TestGeomCoverage(object):
         g4 = shapely.wkt.loads("POLYGON((10 10, 10.1 50, -10 60, 10 80, 80 80, 80 10, 10 10))")
         assert coverage(g1, SRS(4326)) != coverage(g4, SRS(4326))
 
+
 class TestBBOXCoverage(object):
-    def setup(self):
+    def setup_method(self):
         self.coverage = coverage([-10, 10, 80, 80], SRS(4326))
 
     def test_bbox(self):
         assert bbox_equals(self.coverage.bbox, [-10, 10, 80, 80], 0.0001)
 
     def test_geom(self):
-        assert self.coverage.geom == None
+        assert self.coverage.geom is None
 
     def test_contains(self):
         assert self.coverage.contains((15, 15, 20, 20), SRS(4326))
@@ -280,17 +284,15 @@ class TestBBOXCoverage(object):
 
     def test_intersection(self):
         assert (self.coverage.intersection((15, 15, 20, 20), SRS(4326)) ==
-            BBOXCoverage((15, 15, 20, 20), SRS(4326)))
+                BBOXCoverage((15, 15, 20, 20), SRS(4326)))
         assert (self.coverage.intersection((15, 15, 80, 20), SRS(4326)) ==
-            BBOXCoverage((15, 15, 80, 20), SRS(4326)))
+                BBOXCoverage((15, 15, 80, 20), SRS(4326)))
         assert (self.coverage.intersection((9, 10, 20, 20), SRS(4326)) ==
-            BBOXCoverage((9, 10, 20, 20), SRS(4326)))
+                BBOXCoverage((9, 10, 20, 20), SRS(4326)))
         assert (self.coverage.intersection((-30, 10, -8, 70), SRS(4326)) ==
-            BBOXCoverage((-10, 10, -8, 70), SRS(4326)))
-        assert (self.coverage.intersection((-30, 10, -11, 70), SRS(4326)) ==
-            None)
-        assert (self.coverage.intersection((0, 0, 1000, 1000), SRS(900913)) ==
-            None)
+                BBOXCoverage((-10, 10, -8, 70), SRS(4326)))
+        assert (self.coverage.intersection((-30, 10, -11, 70), SRS(4326)) is None)
+        assert (self.coverage.intersection((0, 0, 1000, 1000), SRS(900913)) is None)
         assert bbox_equals(
             self.coverage.intersection((0, 0, 1500000, 1500000), SRS(900913)).bbox,
             (0.0, 10, 13.47472926179282, 13.352207626707813)
@@ -304,11 +306,12 @@ class TestBBOXCoverage(object):
 
 
 class TestUnionCoverage(object):
-    def setup(self):
+    def setup_method(self):
         self.coverage = union_coverage([
             coverage([0, 0, 10, 10], SRS(4326)),
             coverage(shapely.wkt.loads("POLYGON((10 0, 20 0, 20 10, 10 10, 10 0))"), SRS(4326)),
-            coverage(shapely.wkt.loads("POLYGON((-1000000 0, 0 0, 0 1000000, -1000000 1000000, -1000000 0))"), SRS(3857)),
+            coverage(shapely.wkt.loads(
+                "POLYGON((-1000000 0, 0 0, 0 1000000, -1000000 1000000, -1000000 0))"), SRS(3857)),
         ])
 
     def test_bbox(self):
@@ -327,10 +330,11 @@ class TestUnionCoverage(object):
 
 
 class TestDiffCoverage(object):
-    def setup(self):
+    def setup_method(self):
         g1 = coverage(shapely.wkt.loads("POLYGON((-10 0, 20 0, 20 10, -10 10, -10 0))"), SRS(4326))
         g2 = coverage([0, 2, 8, 8], SRS(4326))
-        g3 = coverage(shapely.wkt.loads("POLYGON((-1000000 500000, 0 500000, 0 1000000, -1000000 1000000, -1000000 500000))"), SRS(3857))
+        g3 = coverage(shapely.wkt.loads(
+            "POLYGON((-1000000 500000, 0 500000, 0 1000000, -1000000 1000000, -1000000 500000))"), SRS(3857))
         self.coverage = diff_coverage([g1, g2, g3])
 
     def test_bbox(self):
@@ -339,18 +343,18 @@ class TestDiffCoverage(object):
     def test_contains(self):
         assert self.coverage.contains((0, 0, 1, 1), SRS(4326))
         assert self.coverage.contains((-1100000, 510000, -1050000, 600000), SRS(3857))
-        assert not self.coverage.contains((-1100000, 510000, -990000, 600000), SRS(3857)) # touches # g3
-        assert not self.coverage.contains((4, 4, 5, 5), SRS(4326)) # in g2
+        assert not self.coverage.contains((-1100000, 510000, -990000, 600000), SRS(3857))  # touches # g3
+        assert not self.coverage.contains((4, 4, 5, 5), SRS(4326))  # in g2
 
     def test_intersects(self):
         assert self.coverage.intersects((0, 0, 1, 1), SRS(4326))
         assert self.coverage.intersects((-1100000, 510000, -1050000, 600000), SRS(3857))
-        assert self.coverage.intersects((-1100000, 510000, -990000, 600000), SRS(3857)) # touches # g3
-        assert not self.coverage.intersects((4, 4, 5, 5), SRS(4326)) # in g2
+        assert self.coverage.intersects((-1100000, 510000, -990000, 600000), SRS(3857))  # touches # g3
+        assert not self.coverage.intersects((4, 4, 5, 5), SRS(4326))  # in g2
 
 
 class TestIntersectionCoverage(object):
-    def setup(self):
+    def setup_method(self):
         g1 = coverage(shapely.wkt.loads("POLYGON((0 0, 10 0, 10 10, 0 10, 0 0))"), SRS(4326))
         g2 = coverage([5, 5, 15, 15], SRS(4326))
         self.coverage = intersection_coverage([g1, g2])
@@ -369,7 +373,7 @@ class TestIntersectionCoverage(object):
 
 
 class TestMultiCoverage(object):
-    def setup(self):
+    def setup_method(self):
         # box from 10 10 to 80 80 with small spike/corner to -10 60 (upper left)
         self.geom = shapely.wkt.loads(
             "POLYGON((10 10, 10 50, -10 60, 10 80, 80 80, 80 10, 10 10))")
@@ -409,7 +413,7 @@ class TestMultiCoverage(object):
 
 
 class TestMapExtent(object):
-    def setup(self):
+    def setup_method(self):
         self.extent = MapExtent([-10, 10, 80, 80], SRS(4326))
 
     def test_bbox(self):
@@ -439,10 +443,10 @@ class TestMapExtent(object):
 
     def test_intersection(self):
         assert (DefaultMapExtent().intersection(MapExtent((0, 0, 10, 10), SRS(4326)))
-            == MapExtent((0, 0, 10, 10), SRS(4326)))
+                == MapExtent((0, 0, 10, 10), SRS(4326)))
 
         assert (MapExtent((0, 0, 10, 10), SRS(4326)).intersection(MapExtent((20, 20, 30, 30), SRS(4326)))
-            == None)
+                is None)
 
         sub = MapExtent((0, 0, 10, 10), SRS(4326)).intersection(MapExtent((-1000, -1000, 100000, 100000), SRS(3857)))
         bbox = SRS(3857).transform_bbox_to(SRS(4326), (0, 0, 100000, 100000), 0)
@@ -469,9 +473,15 @@ class TestLoadDatasource(object):
         with TempFile() as fname:
             with open(fname, 'wb') as f:
                 f.write(b'''{"type": "FeatureCollection", "features": [
-                    {"type": "Feature", "geometry": {"type": "Polygon", "coordinates": [[[0, 0], [10, 0], [10, 10], [0, 0]]]} },
-                    {"type": "Feature", "geometry": {"type": "MultiPolygon", "coordinates": [[[[0, 0], [10, 0], [10, 10], [0, 0]]], [[[0, 0], [10, 0], [10, 10], [0, 0]]], [[[0, 0], [10, 0], [10, 10], [0, 0]]]]} },
-                    {"type": "Feature", "geometry": {"type": "Point", "coordinates": [0, 0]} }
+                    {"type": "Feature", "geometry": {"type": "Polygon", "coordinates": [
+                        [[0, 0], [10, 0], [10, 10], [0, 0]]
+                    ]}},
+                    {"type": "Feature", "geometry": {"type": "MultiPolygon", "coordinates": [
+                        [[[0, 0], [10, 0], [10, 10], [0, 0]]],
+                        [[[0, 0], [10, 0], [10, 10], [0, 0]]],
+                        [[[0, 0], [10, 0], [10, 10], [0, 0]]]
+                    ]}},
+                    {"type": "Feature", "geometry": {"type": "Point", "coordinates": [0, 0]}}
                 ]}''')
 
             geoms = load_datasource(fname)
@@ -497,7 +507,7 @@ class TestLoadDatasource(object):
                 f.write(b"4/2/5\n")
                 f.write(b"4/2/6\n")
                 f.write(b"error\n")
-                f.write(b"4/2/1\n") # rest of file is ignored
+                f.write(b"4/2/1\n")  # rest of file is ignored
 
             geoms = load_expire_tiles(fname)
             assert len(geoms) == 2
