@@ -15,7 +15,7 @@
 
 
 from mapproxy.client.tile import TileClient, TileURLTemplate
-from mapproxy.grid import TileGrid
+from mapproxy.grid.tile_grid import TileGrid
 from mapproxy.srs import SRS
 from mapproxy.source.tile import TiledSource
 from mapproxy.source.error import HTTPSourceErrorHandler
@@ -29,7 +29,7 @@ TESTSERVER_URL = ("http://%s:%d" % TEST_SERVER_ADDRESS) + "/%(tms_path)s.png"
 
 class TestTileClientOnError(object):
 
-    def setup(self):
+    def setup_method(self):
         self.grid = TileGrid(SRS(4326), bbox=[-180, -90, 180, 90])
         self.client = TileClient(TileURLTemplate(TESTSERVER_URL))
 
