@@ -22,10 +22,12 @@ from ctypes import c_void_p, c_char_p, c_int
 
 from mapproxy.util.lib import load_library
 
+
 def init_libgdal():
     libgdal = load_library(['libgdal', 'libgdal1', 'gdal110', 'gdal19', 'gdal18', 'gdal17'])
 
-    if not libgdal: return
+    if not libgdal:
+        return
 
     libgdal.OGROpen.argtypes = [c_char_p, c_int, c_void_p]
     libgdal.OGROpen.restype = c_void_p
@@ -37,7 +39,7 @@ def init_libgdal():
         libgdal.CPLGetLastErrorMsg = getattr(libgdal, '_CPLGetLastErrorMsg@0')
 
     if hasattr(libgdal, 'CPLGetLastErrorMsg'):
-        libgdal.CPLGetLastErrorMsg.argtypes	= []
+        libgdal.CPLGetLastErrorMsg.argtypes = []
         libgdal.CPLGetLastErrorMsg.restype = c_char_p
     else:
         libgdal.CPLGetLastErrorMsg = None
@@ -75,16 +77,19 @@ def init_libgdal():
 
     return libgdal
 
+
 class OGRShapeReaderError(Exception):
     pass
 
+
 class CtypesOGRShapeReader(object):
     def __init__(self, datasource):
         self.datasource = datasource
         self._ds = None
 
     def open(self):
-        if self._ds: return
+        if self._ds:
+            return
         self._ds = libgdal.OGROpen(self.datasource.encode(sys.getdefaultencoding()), False, None)
         if self._ds is None:
             msg = None
@@ -95,14 +100,15 @@ class CtypesOGRShapeReader(object):
             raise OGRShapeReaderError(msg)
 
     def wkts(self, where=None):
-        if not self._ds: self.open()
+        if not self._ds:
+            self.open()
 
         if where:
             if not where.lower().startswith('select'):
                 layer = libgdal.OGR_DS_GetLayer(self._ds, 0)
                 layer_def = libgdal.OGR_L_GetLayerDefn(layer)
                 name = libgdal.OGR_FD_GetName(layer_def)
-                where = 'select * from %s where %s' % (name.decode('utf-8'), where)
+                where = 'select * from "%s" where %s' % (name.decode('utf-8'), where)
             layer = libgdal.OGR_DS_ExecuteSQL(self._ds, where.encode('utf-8'), None, None)
         else:
             layer = libgdal.OGR_DS_GetLayer(self._ds, 0)
@@ -118,6 +124,9 @@ class CtypesOGRShapeReader(object):
             if feature is None:
                 break
             geom = libgdal.OGR_F_GetGeometryRef(feature)
+            if geom is None:
+                libgdal.OGR_F_Destroy(feature)
+                continue
             res = c_char_p()
             libgdal.OGR_G_ExportToWkt(geom, ctypes.byref(res))
             yield res.value
@@ -142,7 +151,8 @@ class OSGeoOGRShapeReader(object):
         self._ds = None
 
     def open(self):
-        if self._ds: return
+        if self._ds:
+            return
         self._ds = ogr.Open(self.datasource, False)
         if self._ds is None:
             msg = gdal.GetLastErrorMsg()
@@ -151,13 +161,14 @@ class OSGeoOGRShapeReader(object):
             raise OGRShapeReaderError(msg)
 
     def wkts(self, where=None):
-        if not self._ds: self.open()
+        if not self._ds:
+            self.open()
 
         if where:
             if not where.lower().startswith('select'):
                 layer = self._ds.GetLayerByIndex(0)
                 name = layer.GetName()
-                where = 'select * from %s where %s' % (name, where)
+                where = 'select * from "%s" where %s' % (name, where)
             layer = self._ds.ExecuteSQL(where)
         else:
             layer = self._ds.GetLayerByIndex(0)
@@ -179,22 +190,30 @@ class OSGeoOGRShapeReader(object):
 
 
 ogr = gdal = None
+
+
 def try_osgeoogr_import():
     global ogr, gdal
     try:
-        from osgeo import ogr; ogr
-        from osgeo import gdal; gdal
+        from osgeo import ogr
+        ogr
+        from osgeo import gdal
+        gdal
     except ImportError:
         return
     return OSGeoOGRShapeReader
 
+
 libgdal = None
+
+
 def try_libogr_import():
     global libgdal
     libgdal = init_libgdal()
     if libgdal is not None:
         return CtypesOGRShapeReader
 
+
 ogr_imports = []
 if 'MAPPROXY_USE_OSGEOOGR' in os.environ:
     ogr_imports = [try_osgeoogr_import]
